" -----------------------------------------------------------------------------
" THE BEST VIMRC EVER:
" -----------------------------------------------------------------------------

" Plugin Manager: {{{
call plug#begin('~/.config/nvim/plugged')
Plug 'neovim/nvim-lspconfig'
Plug 'neoclide/coc.nvim', {'branch': 'release'}                " Autocompletion
Plug 'nvim-treesitter/nvim-treesitter', {'do': ':TSUpdate'}    " Language parser for outline (aerial)
Plug 'scrooloose/nerdcommenter'                                " Programming language specific comments
Plug 'kyazdani42/nvim-web-devicons'                            " optional, for file icons
Plug 'terryma/vim-multiple-cursors'                            " Multicursor
Plug 'lervag/vimtex'                                           " Latex in vim
" Plug 'vim-airline/vim-airline'                                 " Fancy line at bottom
Plug 'nvim-lualine/lualine.nvim'                               " Fancy line at the bottom
" Plug 'stevearc/aerial.nvim'                                    " Code outline and navigation
Plug 'simrat39/symbols-outline.nvim'
Plug 'kyazdani42/nvim-web-devicons'                            " optional, for file icons
Plug 'kyazdani42/nvim-tree.lua'                                " File browser
Plug 'iamcco/markdown-preview.nvim', { 'do': 'cd app && yarn install' } " Markdown preview
" Appearence:
" :colortheme ...
Plug 'morhetz/gruvbox'                                         " Gruvbox theme 
Plug 'EdenEast/nightfox.nvim'                                  " Nightfox theme
Plug 'altercation/vim-colors-solarized'                        " Solarized theme
Plug 'vim-airline/vim-airline-themes'
" :AirlineTheme ...
call plug#end() 
" }}}

" Custom Neovim Settings: {{{
    let g:mapleader = ','           " Map the leader key
    let g:maplocalleader = ','
    set foldmethod=marker
    set ttimeout
    set ttimeoutlen=100
    set timeoutlen=3000
    set backspace=indent,eol,start  " Enables backspace
    syntax on
   set nu rnu                      " Sets relative + absolute numbering
    set encoding=utf-8              " Change output encoding from terminal
    set splitbelow                  " All horizontal splits are below
    set splitright                  " All vertical slits are to the right
    set textwidth=0                 " Specifies where to force a line break is, 0 = no linebreak
    set tabstop=3                   " number of spaces inserted when the tab key is pressed
    set softtabstop=3               " interpret multiple spaces as tabstops
    set shiftwidth=3			    " width for autoindents
    set expandtab				    " convert tabs to white spaces
    set smarttab				    " Indent stuff
    set autoindent				    " Use the indent from last line if a new line is added
    set smartindent				    " Smartly indent on new line
    set fileformat=unix			    " Sets the fileformat
    filetype indent on              " load filetype-specific indent files
    set lazyredraw                  " redraw only when we need to.
    set showmatch                   " highlight matching [{()}]
    set clipboard=unnamedplus      " clipboard for nvim
    set incsearch                   " search as characters are entered
    set nowrapscan                  " Stop serach at end of file
    set hlsearch                    " highlight matches
    set noswapfile                  " Turn Off Swap Files
    set nobackup                    " Turn Off Swap Files
    set nowb                        " Turn Off Swap Files
    set mouse=a                     " set middle mouse for insert
    set termguicolors
    " Disables the omnicompletion back to original error
    set noshowmode
    try
      set shortmess+=c
    catch /^Vim\%((\a\+)\)\=:E539: Illegal character/
      autocmd MyAutoCmd VimEnter *
	    \ highlight ModeMsg guifg=bg guibg=bg |
	    \ highlight Question guifg=bg guibg=bg
    endtry
    " Jumpt to last position
    autocmd BufReadPost *
		\if @% !~# '\.git[\/\\]COMMIT_EDITMSG$' && line("'\"") > 1 && line("'\"") <= line("$") |
		\exe "normal! g`\"" |
		\endif
    colorscheme carbonfox
" }}}

" Custom Neovim Mappings: {{{

    "         COMMANDS                 MODES:
    " :map   :noremap  :unmap     Normal, Visual, Select, Operator-pending
    " :nmap  :nnoremap :nunmap    Normal
    " :vmap  :vnoremap :vunmap    Visual and Select
    " :smap  :snoremap :sunmap    Select
    " :xmap  :xnoremap :xunmap    Visual
    " :omap  :onoremap :ounmap    Operator-pending
    " :map!  :noremap! :unmap!    Insert and Command-line
    " :imap  :inoremap :iunmap    Insert
    " :lmap  :lnoremap :lunmap    Insert, Command-line, Lang-Arg
    " :cmap  :cnoremap :cunmap    Command-line
    " :tmap  :tnoremap :tunmap    Terminal

   " inoremap " ""<left>
   " inoremap ' ''<left>
   " inoremap ( ()<left>
   " inoremap [ []<left>
   " inoremap { {}<left>
   " inoremap {<CR> {<CR>}<ESC>O
   " inoremap   {<CR>};<ESC>O
   "
   " Escape from terminal mode
   :tmap <Esc> <C-\><C-n>

   " resize split window
    imap <C-A-h>         <Esc>:exe "vertical resize +5"<CR>       " increase by -5
    nmap <C-A-h>         :exe "vertical resize +5"<CR>
    imap <C-A-l>       <Esc>:exe "vertical resize -5"<CR>       " decrease by 5
    nmap <C-A-l>       :exe "vertical resize -5"<CR>
    imap <C-A-j>         <Esc>:exe "resize +5"<CR>       " increase by -5
    nmap <C-A-j>         :exe "resize +5"<CR>
    imap <C-A-k>       <Esc>:exe "resize -5"<CR>       " decrease by 5
    nmap <C-A-k>       :exe "resize -5"<CR>
    " move line
    imap <A-k>         <Esc>:m-2<CR>
    nmap <A-k>         :m-2<CR>
    imap <A-j>       <Esc>:m+<CR>
    nmap <A-j>       :m+<CR>
    " save (write)
    imap <C-s>          <Esc>:w<CR>i
    nmap <C-s>          :w<CR>
   " undo
   imap <C-z>			<Esc>u<CR>i
   nmap <C-z>			u<CR>
    "redo
    imap <C-y>			<Esc>:redo<CR>i
   nmap <C-y>			:redo<CR>
    " tabs
    nmap <C-t>          :tabe<CR>
    imap <C-t>          <Esc>:tabe<CR>i
    imap <C-PageUp>     <Esc>gt<CR>i
    nmap <C-PageUp>     gt<CR>
    imap <C-PageDown>   <Esc>gT<CR>i
    nmap <C-PageDown>   gT<CR>
    "
" }}}

" Plugin Settings: {{{
" Coc {{{
" Some servers have issues with backup files, see #649.
set nobackup
set nowritebackup
" Having longer updatetime (default is 4000 ms = 4 s) leads to noticeable
" delays and poor user experience.
set updatetime=300
" Always show the signcolumn, otherwise it would shift the text each time
" diagnostics appear/become resolved.
set signcolumn=yes

function! CheckBackspace() abort
  let col = col('.') - 1
  return !col || getline('.')[col - 1]  =~# '\s'
endfunction

function! ShowDocumentation()
  if CocAction('hasProvider', 'hover')
    call CocActionAsync('doHover':)
  else
    call feedkeys('K', 'in')
  endif
endfunction
" Add `:Format` command to format current buffer.
command! -nargs=0 Format :call CocActionAsync('format')
" Add `:Fold` command to fold current buffer.
command! -nargs=? Fold :call     CocAction('fold', <f-args>)
" Add `:OR` command for organize imports of the current buffer.
command! -nargs=0 OR   :call     CocActionAsync('runCommand', 'editor.action.organizeImport')
" Add (Neo)Vim's native statusline support.
" NOTE: Please see `:h coc-status` for integrations with external plugins that
" provide custom statusline: lightline.vim, vim-airline.
set statusline^=%{coc#status()}%{get(b:,'coc_current_function','')}
" }}}
" Airline: {{{
let g:airline_powerline_fonts = 1       " enable powerline fonts
let g:airline_theme = "base16_vim"      " availables themes can be looked up at https://github.com/vim-airline/vim-airline-themes/blob/master/doc/airline-themes.txt
" }}}
" Gruvbox {{{
let g:gruvbox_italic = 1                " enable italic text
let g:gruvbox_italicize_comments = 1    " enable italic text for comments
let g:gruvbox_bold = 1                  " enable bold text
let g:gruvbox_termcolors = 16           " use 16-bit color palette (default: 256)
let g:gruvbox_contrast_dark = "hard"    " set soft contrast for dark the dark mode
let g:gruvbox_number_column = "bg0"    " set the number column background color
" }}}
" Magma {{{
let g:magma_automatically_open_output = v:false
let g:magma_image_provider = "kitty"
let g:magma_show_mimetype_debug = v:true
" }}}
" Nerdcommenter {{{
let g:NERDCreateDefaultMappings = 1             " Create default mappings
let g:NERDSpaceDelims = 1                       " Add spaces after comment delimiters by default
let g:NERDCompactSexyComs = 1                   " Use compact syntax for prettified multi-line comments
let g:NERDDefaultAlign = 'left'                 " Align line-wise comment delimiters flush left instead of following code indentation
let g:NERDAltDelims_java = 1                    " Set a language to use its alternate delimiters by default
let g:NERDCustomDelimiters = { 'c': { 'left': '/**','right': '*/' } }   " Add your own custom formats or override the defaults
let g:NERDCommentEmptyLines = 1                 " Allow commenting and inverting empty lines (useful when commenting a region)
let g:NERDTrimTrailingWhitespace = 1            " Enable trimming of trailing whitespace when uncommenting
let g:NERDToggleCheckAllLines = 1               " Enable NERDCommenterToggle to check all selected lines is commented or not
" }}}
" Vimtex {{{
let b:vimtex_main = 'main.tex'
let g:tex_flavor = "latex"
let g:vimtex_compiler_progname = 'nvr'
let g:vimtex_complete_bib = {'simple': 0}
let g:vimtex_complete_close_braces = 1
let g:vimtex_complete_enabled = 1
let g:vimtex_complete_smart_case = 1
let g:vimtex_indent_enabled = 1
let g:vimtex_fold_automatic = 1
" Viewer options: One may configure the viewer either by specifying a built-in
" viewer method:
let g:vimtex_view_method = 'zathura'
" Or with a generic interface:
" let g:vimtex_view_general_viewer = 'okular'
" let g:vimtex_view_general_viewer = 'zathura'
let g:vimtex_view_general_options = '--unique file:@pdf\#src:@line@tex'
" table of content
let g:vimtex_toc_enabled = 1
let g:vimtex_toc_config = {
    \ 'name' : 'TOC',
    \ 'layers' : ['content', 'todo', 'include'],
    \ 'resize' : 1,
    \ 'split_width' : 50,
    \ 'todo_sorted' : 0,
    \ 'show_help' : 1,
    \ 'show_numbers' : 1,
    \ 'mode' : 2,
    \}
" }}}
" MarkdownPreview {{{
" set to 1, nvim will open the preview window after entering the markdown buffer
" default: 0
let g:mkdp_auto_start = 0
" set to 1, the nvim will auto close current preview window when change
" from markdown buffer to another buffer
" default: 1
let g:mkdp_auto_close = 1
" set to 1, the vim will refresh markdown when save the buffer or
" leave from insert mode, default 0 is auto refresh markdown as you edit or
" move the cursor
" default: 0
let g:mkdp_refresh_slow = 1
" set to 1, the MarkdownPreview command can be use for all files,
" by default it can be use in markdown file
" default: 0
let g:mkdp_command_for_global = 0
" set to 1, preview server available to others in your network
" by default, the server listens on localhost (127.0.0.1)
" default: 0
let g:mkdp_open_to_the_world = 0
" use custom IP to open preview page
" useful when you work in remote vim and preview on local browser
" more detail see: https://github.com/iamcco/markdown-preview.nvim/pull/9
" default empty
let g:mkdp_open_ip = ''
" specify browser to open preview page
" for path with space
" valid: `/path/with\ space/xxx`
" invalid: `/path/with\\ space/xxx`
" default: ''
let g:mkdp_browser = ''
" set to 1, echo preview page url in command line when open preview page
" default is 0
let g:mkdp_echo_preview_url = 0
" a custom vim function name to open preview page
" this function will receive url as param
" default is empty
let g:mkdp_browserfunc = ''
" options for markdown render
" mkit: markdown-it options for render
" katex: katex options for math
" uml: markdown-it-plantuml options
" maid: mermaid options
" disable_sync_scroll: if disable sync scroll, default 0
" sync_scroll_type: 'middle', 'top' or 'relative', default value is 'middle'
"   middle: mean the cursor position alway show at the middle of the preview page
"   top: mean the vim top viewport alway show at the top of the preview page
"   relative: mean the cursor position alway show at the relative positon of the preview page
" hide_yaml_meta: if hide yaml metadata, default is 1
" sequence_diagrams: js-sequence-diagrams options
" content_editable: if enable content editable for preview page, default: v:false
" disable_filename: if disable filename header for preview page, default: 0
let g:mkdp_preview_options = {
    \ 'mkit': {},
    \ 'katex': {},
    \ 'uml': {},
    \ 'maid': {},
    \ 'disable_sync_scroll': 0,
    \ 'sync_scroll_type': 'middle',
    \ 'hide_yaml_meta': 1,
    \ 'sequence_diagrams': {},
    \ 'flowchart_diagrams': {},
    \ 'content_editable': v:false,
    \ 'disable_filename': 0,
    \ 'toc': {}
    \ }
" use a custom markdown style must be absolute path
" like '/Users/username/markdown.css' or expand('~/markdown.css')
let g:mkdp_markdown_css = ''
" use a custom highlight style must absolute path
" like '/Users/username/highlight.css' or expand('~/highlight.css')
let g:mkdp_highlight_css = ''
" use a custom port to start server or empty for random
let g:mkdp_port = ''
" preview page title
" ${name} will be replace with the file name
let g:mkdp_page_title = '「${name}」'
" recognized filetypes
" these filetypes will have MarkdownPreview... commands
let g:mkdp_filetypes = ['markdown']
" set default theme (dark or light)
" By default the theme is define according to the preferences of the system
let g:mkdp_theme = 'dark'
" }}}
" }}}

" Plugin Mappings: {{{
" Coc {{{
" Use tab for trigger completion with characters ahead and navigate.
" NOTE: Use command ':verbose imap <tab>' to make sure tab is not mapped by
" other plugin before putting this into your config.
inoremap <silent><expr> <TAB>
      \ coc#pum#visible() ? coc#pum#next(1):
      \ CheckBackspace() ? "\<Tab>" :
      \ coc#refresh()
inoremap <expr><S-TAB> coc#pum#visible() ? coc#pum#prev(1) : "\<C-h>"
" Make <CR> to accept selected completion item or notify coc.nvim to format
" <C-g>u breaks current undo, please make your own choice.
noremap <silent><expr> <CR> coc#pum#visible() ? coc#pum#confirm(): "\<C-g>u\<CR>\<c-r>=coc#on_enter()\<CR>"
" Use <c-space> to trigger completion.
if has('nvim')
  inoremap <silent><expr> <c-space> coc#refresh()
else
  inoremap <silent><expr> <c-@> coc#refresh()
endif
" Use `[g` and `]g` to navigate diagnostics
" Use `:CocDiagnostics` to get all diagnostics of current buffer in location list.
nmap <silent> [g <Plug>(coc-diagnostic-prev)
nmap <silent> ]g <Plug>(coc-diagnostic-next)
" GoTo code navigation.
nmap <silent> gd <Plug>(coc-definition)
nmap <silent> gy <Plug>(coc-type-definition)
nmap <silent> gi <Plug>(coc-implementation)
nmap <silent> gr <Plug>(coc-references)
" Use K to show documentation in preview window.
nnoremap <silent> K :call ShowDocumentation()<CR>
" Highlight the symbol and its references when holding the cursor.
autocmd CursorHold * silent call CocActionAsync('highlight')
" Symbol renaming.
nmap <leader>rn <Plug>(coc-rename)
" Formatting selected code.
xmap <leader>f  <Plug>(coc-format-selected)
nmap <leader>f  <Plug>(coc-format-selected)
"
augroup mygroup
  autocmd!
  " Setup formatexpr specified filetype(s).
  autocmd FileType typescript,json setl formatexpr=CocAction('formatSelected')
  " Update signature help on jump placeholder.
  autocmd User CocJumpPlaceholder call CocActionAsync('showSignatureHelp')
augroup end
" Applying codeAction to the selected region.
" Example: `<leader>aap` for current paragraph
xmap <leader>a  <Plug>(coc-codeaction-selected)
nmap <leader>a  <Plug>(coc-codeaction-selected)
" Remap keys for applying codeAction to the current buffer.
nmap <leader>ac  <Plug>(coc-codeaction)
" Apply AutoFix to problem on the current line.
nmap <leader>qf  <Plug>(coc-fix-current)
" Run the Code Lens action on the current line.
nmap <leader>cl  <Plug>(coc-codelens-action)
" Map function and class text objects
" NOTE: Requires 'textDocument.documentSymbol' support from the language server.
xmap if <Plug>(coc-funcobj-i)
omap if <Plug>(coc-funcobj-i)
xmap af <Plug>(coc-funcobj-a)
omap af <Plug>(coc-funcobj-a)
xmap ic <Plug>(coc-classobj-i)
omap ic <Plug>(coc-classobj-i)
xmap ac <Plug>(coc-classobj-a)
omap ac <Plug>(coc-classobj-a)
" Remap <C-f> and <C-b> for scroll float windows/popups.
if has('nvim-0.4.0') || has('patch-8.2.0750')
  nnoremap <silent><nowait><expr> <C-f> coc#float#has_scroll() ? coc#float#scroll(1) : "\<C-f>"
  nnoremap <silent><nowait><expr> <C-b> coc#float#has_scroll() ? coc#float#scroll(0) : "\<C-b>"
  inoremap <silent><nowait><expr> <C-f> coc#float#has_scroll() ? "\<c-r>=coc#float#scroll(1)\<cr>" : "\<Right>"
  inoremap <silent><nowait><expr> <C-b> coc#float#has_scroll() ? "\<c-r>=coc#float#scroll(0)\<cr>" : "\<Left>"
  vnoremap <silent><nowait><expr> <C-f> coc#float#has_scroll() ? coc#float#scroll(1) : "\<C-f>"
  vnoremap <silent><nowait><expr> <C-b> coc#float#has_scroll() ? coc#float#scroll(0) : "\<C-b>"
endif
" Use CTRL-S for selections ranges.
" Requires 'textDocument/selectionRange' support of language server.
nmap <silent> <C-s> <Plug>(coc-range-select)
xmap <silent> <C-s> <Plug>(coc-range-select)
" Mappings for CoCList
" Show all diagnostics.
nnoremap <silent><nowait> <space>a  :<C-u>CocList diagnostics<cr>
" Manage extensions.
nnoremap <silent><nowait> <space>e  :<C-u>CocList extensions<cr>
" Show commands.
nnoremap <silent><nowait> <space>c  :<C-u>CocList commands<cr>
" Find symbol of current document.
nnoremap <silent><nowait> <space>o  :<C-u>CocList outline<cr>
" Search workspace symbols.
nnoremap <silent><nowait> <space>s  :<C-u>CocList -I symbols<cr>
" Do default action for next item.
nnoremap <silent><nowait> <space>j  :<C-u>CocNext<CR>
" Do default action for previous item.
nnoremap <silent><nowait> <space>k  :<C-u>CocPrev<CR>
" Resume latest coc list.
nnoremap <silent><nowait> <space>p  :<C-u>CocListResume<CR>
" }}}
" Magma {{{
nnoremap <silent><expr> <LocalLeader>r  :MagmaEvaluateOperator<CR>
nnoremap <silent>       <LocalLeader>rr :MagmaEvaluateLine<CR>
xnoremap <silent>       <LocalLeader>r  :<C-u>MagmaEvaluateVisual<CR>
nnoremap <silent>       <LocalLeader>rc :MagmaReevaluateCell<CR>
nnoremap <silent>       <LocalLeader>rd :MagmaDelete<CR>
nnoremap <silent>       <LocalLeader>ro :MagmaShowOutput<CR>
" }}}
" Multicursor {{{
let g:multi_cursor_use_default_mapping=0
" Default mapping
let g:multi_cursor_start_word_key      = '<C-n>'
let g:multi_cursor_select_all_word_key = '<A-n>'
let g:multi_cursor_start_key           = 'g<C-n>'
let g:multi_cursor_select_all_key      = 'g<A-n>'
let g:multi_cursor_next_key            = '<C-n>'
let g:multi_cursor_prev_key            = '<C-p>'
let g:multi_cursor_skip_key            = '<C-x>'
let g:multi_cursor_quit_key            = '<Esc>'
" }}}
" nvim-tree {{{
imap <C-b>          <Esc>:NvimTreeToggle<CR>
nmap <C-b>          :NvimTreeToggle<CR>
" }}}
" SymbolsOutline {{{
nmap <Leader>o :SymbolsOutline<CR>

" }}}
" }}}

" lua soource files: {{{
" Set up laguage servers
lua require'lspconfig'.pyright.setup{}
lua require'lspconfig'.clangd.setup{}
" lua require('lspconfig').ccls.setup{}
lua require('_aerial')
lua require('_symbols-outline')
lua require('_treesitter')
lua require("nvim-tree").setup()
lua require('_lualine')
" }}}
